require 'fastlane/action'
require_relative '../helper/android_cd_helper'

module Fastlane
  module Actions
    class BuildAndDeployAction < Action
      def self.run(params)
        # Check parameters
        unless Helper::AndroidCdHelper.is_set(params[:beta_type])
          raise ArgumentError, "❌ Parameters beta_type cannot be null"
        end

        UI.message("⌛️ Building and deploying to Google Play Store in #{params[:beta_type]}..")

        # Decrypt the keys archive and Extract the keys archive
        Helper::AndroidCdHelper.decrypt_android_keys('.')
        UI.message("👉🏼 Credentials decrypted")

        # Build the Android App Bundle
        other_action.flutter_build(
          build: 'appbundle',
          build_args: params[:build_args]
        )

        UI.message("👉🏼 App built")

        # Upload the Android App Bundle to the Play Store
        begin
          other_action.upload_to_play_store(
            track: params[:beta_type],
            json_key: './service_account_key.json',
            aab: '../build/app/outputs/bundle/release/app-release.aab',
            skip_upload_metadata: true,
            skip_upload_images: true,
            skip_upload_screenshots: true,
            release_status: "completed"
          )
        rescue StandardError => e
          UI.message("👉🏼 Unable to upload as completed; try to upload it in draft status")
          other_action.upload_to_play_store(
            track: params[:beta_type],
            json_key: './service_account_key.json',
            aab: '../build/app/outputs/bundle/release/app-release.aab',
            skip_upload_metadata: true,
            skip_upload_images: true,
            skip_upload_screenshots: true,
            release_status: "draft"
          )
        end
        

        # Delete artifacts files
        artifacts = ['android_keys.zip', 'key.jks', 'key.properties', 'service_account_key.json']
        Helper::AndroidCdHelper.delete_artifacts(artifacts)

        UI.success('🍺 Successfully build & deploy appbundle to Google Play Store')
      end

      def self.description
        "Google Play Store deployment plugin for Fastlane, simplifying the build and deployment process to internal, beta, alpha, and production channels, and promoting builds for testing."
      end

      def self.authors
        ["SAS Wyatt Studio"]
      end

      def self.return_value
      end

      def self.details
        "The Fastlane Google Play Store deployment action streamlines the build and deployment process to internal, beta, alpha, and production channels, simplifying the process of distributing builds for testing. With its advanced promotion functionality, it also enables easy promotion of builds to higher beta testing phases, helping you get your app to market faster."
      end

      def self.available_options
        [
          FastlaneCore::ConfigItem.new(key: :beta_type,
            env_name: "ANDROID_CD_TEST_TYPE",
            optional: false,
            description: "Type of test (production, beta, alpha, internal)"),
          FastlaneCore::ConfigItem.new(
            key: :build_args,
            description: 'An array of extra arguments for "flutter build"',
            optional: true,
            type: Array
          )
        ]
      end

      def self.is_supported?(platform)
        # Restrict the use of the plugin to Android.
        [:android].include?(platform)
      end
    end
  end
end
